from fastapi import FastAPI
from fastapi.testclient import TestClient
from typing import Any
import air


def test_TagResponse_obj():
    """Test the TagResponse class."""
    app = FastAPI()

    @app.get("/test")
    def test_endpoint():
        return air.TagResponse(air.H1("Hello, World!"))

    client = TestClient(app)
    response = client.get("/test")

    assert response.status_code == 200
    assert response.headers["content-type"] == "text/html; charset=utf-8"
    assert response.text == "<h1>Hello, World!</h1>"


def test_TagResponse_type():
    """Test the TagResponse class."""

    app = FastAPI()

    @app.get("/test", response_class=air.TagResponse)
    def test_endpoint():
        return air.Main(
            air.H1("Hello, clean HTML response!"),
            air.P("This is a paragraph in the response."),
        )

    client = TestClient(app)
    response = client.get("/test")

    assert response.status_code == 200
    assert response.headers["content-type"] == "text/html; charset=utf-8"
    assert (
        response.text
        == "<main><h1>Hello, clean HTML response!</h1><p>This is a paragraph in the response.</p></main>"
    )


def test_TagResponse_html():
    """Test the TagResponse class."""

    app = FastAPI()

    @app.get("/test", response_class=air.TagResponse)
    def test_endpoint():
        return air.Html(
            air.Head(),
            air.Body(
                air.Main(
                    air.H1("Hello, clean HTML response!"),
                    air.P("This is a paragraph in the response."),
                )
            ),
        )

    client = TestClient(app)
    response = client.get("/test")

    assert response.status_code == 200
    assert response.headers["content-type"] == "text/html; charset=utf-8"
    assert (
        response.text
        == "<!doctype html><html><head></head><body><main><h1>Hello, clean HTML response!</h1><p>This is a paragraph in the response.</p></main></body></html>"
    )


def test_strings_and_tag_children():
    app = FastAPI()

    @app.get("/test", response_class=air.TagResponse)
    def test_endpoint():
        return air.Html(
            air.Body(air.P("This isn't a ", air.Strong("cut off"), " sentence"))
        )

    client = TestClient(app)
    response = client.get("/test")
    assert response.status_code == 200
    assert response.headers["content-type"] == "text/html; charset=utf-8"
    assert (
        response.text
        == "<!doctype html><html><body><p>This isn&#x27;t a <strong>cut off</strong> sentence</p></body></html>"
    )


def test_custom_name_in_response():
    app = FastAPI()

    def Card(sentence):
        return air.Article(air.Header("Header"), sentence, air.Footer("Footer"))

    @app.get("/test", response_class=air.TagResponse)
    def test_endpoint():
        return Card("This is a sentence")

    client = TestClient(app)
    response = client.get("/test")
    assert response.status_code == 200
    assert response.headers["content-type"] == "text/html; charset=utf-8"
    assert (
        response.text
        == "<article><header>Header</header>This is a sentence<footer>Footer</footer></article>"
    )


def test_TagResponse_with_layout_strings():
    class CustomLayoutResponse(air.TagResponse):
        def render(self, content: Any) -> bytes:
            content = super().render(content)
            return f"<html><body><h1>Custom Layout</h1>{content}</body></html>".encode(
                "utf-8"
            )

    app = FastAPI()

    @app.get("/test", response_class=CustomLayoutResponse)
    def test_endpoint():
        return air.Main(air.H2("Hello, World!"))

    client = TestClient(app)
    response = client.get("/test")

    assert response.status_code == 200
    assert response.headers["content-type"] == "text/html; charset=utf-8"
    assert (
        response.text
        == "<html><body><h1>Custom Layout</h1>b'<main><h2>Hello, World!</h2></main>'</body></html>"
    )


def test_TagResponse_with_layout_names():
    class CustomLayoutResponse(air.TagResponse):
        def render(self, content: Any) -> bytes:
            content = super().render(content).decode("utf-8")
            return air.Html(air.RawHTML(content)).render().encode("utf-8")

    app = FastAPI()

    @app.get("/test", response_class=CustomLayoutResponse)
    def test_endpoint():
        return air.Body(air.Main(air.H1("Hello, World!")))

    client = TestClient(app)
    response = client.get("/test")

    assert response.status_code == 200
    assert response.headers["content-type"] == "text/html; charset=utf-8"
    assert (
        response.text
        == "<!doctype html><html><body><main><h1>Hello, World!</h1></main></body></html>"
    )


def test_AirResponse():
    """Test the AirResponse class."""
    app = FastAPI()

    @app.get("/test_tag", response_class=air.AirResponse)
    def test_tag_endpoint():
        return air.H1("Hello, World!")

    @app.get("/test_html", response_class=air.AirResponse)
    def test_html_endpoint():
        return "<h1>Hello, World!</h1>"

    client = TestClient(app)
    response = client.get("/test_tag")

    assert response.status_code == 200
    assert response.headers["content-type"] == "text/html; charset=utf-8"
    assert response.text == "<h1>Hello, World!</h1>"

    response = client.get("/test_html")

    assert response.status_code == 200
    assert response.headers["content-type"] == "text/html; charset=utf-8"
    assert response.text == "<h1>Hello, World!</h1>"



from fastapi import Request
from fastapi.templating import Jinja2Templates
from typing import Any


class Jinja2Renderer:
    """Template renderer to make Jinja2 easier in Air.

    Args:
        directory: Template directory

    Example:
        >>> # Instantiate the render callable
        >>> render = TemplateRenderer('templates')
        >>>
        >>> # Use for returning Jinja2 from views
        >>> @app.get('/')
        >>> async def home(request: Request):
        >>> return render(
        ...     request,
        ...     'home.html',
        ...     context={'id': 5}
        ... )
    """

    def __init__(self, directory: str):
        """Initialize with template directory path"""
        self.templates = Jinja2Templates(directory=directory)

    def __call__(
        self, request: Request, name: str, context: dict[Any, Any] | None = None
    ):
        """Render template with request and context"""
        if context is None:
            context = {}
        return self.templates.TemplateResponse(
            request=request, name=name, context=context
        )



"""Adds s-expression HTML tags to air."""

from functools import cached_property
import html


def _fix_k(k):
    # This function originated in fastcore.
    return k if k == "_" else k.lstrip("_").replace("_", "-")


_specials = set("@.-!~:[](){}$%^&*+=|/?<>,`")


def attrmap(o):
    """Converts shims for class and for such as _class, cls, _for, fr into
    their HTML implementations
    """
    # This function originated in fastcore.
    if _specials & set(o):
        return o
    o = dict(
        htmlClass="class",
        cls="class",
        _class="class",
        klass="class",
        _for="for",
        fr="for",
        htmlFor="for",
    ).get(o, o)
    return _fix_k(o)


class Tag:
    def __init__(self, *children, **kwargs):
        """Sets four attributes, name, module, children, and attrs.
        These are important for Starlette view responses, as nested objects
        get auto-serialized to JSON and need to be rebuilt. Without
        the values of these attributes, the object reconstruction can't occur"""
        self._name = self.__class__.__name__
        self._module = self.__class__.__module__
        self._children, self._attrs = children, kwargs

    @property
    def name(self) -> str:
        return self._name.lower()

    @property
    def attrs(self) -> str:
        if not self._attrs:
            return ""
        return " " + " ".join(f'{attrmap(k)}="{v}"' for k, v in self._attrs.items())

    @cached_property
    def children(self):
        elements = []
        for child in self._children:
            if isinstance(child, Tag):
                elements.append(child.render())
            elif isinstance(child, str):
                elements.append(html.escape(child))
            elif isinstance(child, int):
                elements.append(str(child))
            else:
                # TODO: Produce a better error message
                msg = f"Unsupported child type: {type(child)}"
                msg += f"\n in tag {self.name}"
                msg += f"\n child {child}"
                msg += f"\n data {self.__dict__}"
                raise TypeError(msg)
        return "".join(elements)

    def render(self) -> str:
        return f"<{self.name}{self.attrs}>{self.children}</{self.name}>"


class CaseTag(Tag):
    """This is for case-sensitive tags like those used in SVG generation."""

    @property
    def name(self) -> str:
        return self._name[0].lower() + self._name[1:]


# Special tags


class Html(Tag):
    """Defines the root of an HTML document"""

    def render(self) -> str:
        return f"""<!doctype html><html{self.attrs}>{self.children}</html>"""


class RawHTML(Tag):
    """Custom tag for rendering raw HTML content without escaping.

    Args:
        html_string: A single string containing raw HTML to render

    Raises:
        TypeError: If non-string content is provided
        ValueError: If multiple arguments are provided

    Example:
        >>> RawHTML('<strong>Bold</strong> text')
        '<strong>Bold</strong> text'

        >>> # Use with other tags
        >>> Div(
        ...     P("Safe content"),
        ...     RawHTML('<hr class="divider">'),
        ...     P("More safe content")
        ... )
    """

    def __init__(self, *args, **kwargs):
        """Initialize RawHTML with a single string argument.

        Args:
            *args: Should be exactly one string argument
            **kwargs: Ignored (for consistency with Tag interface)
        """
        if len(args) > 1:
            raise ValueError("RawHTML accepts only one string argument")

        html_string = args[0] if args else ""

        if not isinstance(html_string, str):
            raise TypeError("RawHTML only accepts string content")

        super().__init__(html_string)

    def render(self) -> str:
        """Render the raw HTML string without escaping."""
        return self._children[0] if self._children else ""


# Stock tags


class A(Tag):
    """Defines a hyperlink"""

    pass


class Abbr(Tag):
    """Defines an abbreviation or an acronym"""

    pass


class Address(Tag):
    """Defines contact information for the author/owner of a document"""

    pass


class Area(Tag):
    """Defines an area inside an image map"""

    pass


class Article(Tag):
    """Defines an article"""

    pass


class Aside(Tag):
    """Defines content aside from the page content"""

    pass


class Audio(Tag):
    """Defines embedded sound content"""

    pass


class B(Tag):
    """Defines bold text"""

    pass


class Base(Tag):
    """Specifies the base URL/target for all relative URLs in a document"""

    pass


class Bdi(Tag):
    """Isolates a part of text that might be formatted in a different direction from other text outside it"""

    pass


class Bdo(Tag):
    """Overrides the current text direction"""

    pass


class Blockquote(Tag):
    """Defines a section that is quoted from another source"""

    pass


class Body(Tag):
    """Defines the document's body"""

    pass


class Br(Tag):
    """Defines a single line break"""

    pass


class Button(Tag):
    """Defines a clickable button"""

    pass


class Canvas(Tag):
    """Used to draw graphics, on the fly, via scripting (usually JavaScript)"""

    pass


class Caption(Tag):
    """Defines a table caption"""

    pass


class Cite(Tag):
    """Defines the title of a work"""

    pass


class Code(Tag):
    """Defines a piece of computer code"""

    pass


class Col(Tag):
    """Specifies column properties for each column within a <colgroup> element"""

    pass


class Colgroup(Tag):
    """Specifies a group of one or more columns in a table for formatting"""

    pass


class Data(Tag):
    """Adds a machine-readable translation of a given content"""

    pass


class Datalist(Tag):
    """Specifies a list of pre-defined options for input controls"""

    pass


class Dd(Tag):
    """Defines a description/value of a term in a description list"""

    pass


class Del(Tag):
    """Defines text that has been deleted from a document"""

    pass


class Details(Tag):
    """Defines additional details that the user can view or hide"""

    pass


class Dfn(Tag):
    """Specifies a term that is going to be defined within the content"""

    pass


class Dialog(Tag):
    """Defines a dialog box or window"""

    pass


class Div(Tag):
    """Defines a section in a document"""

    pass


class Dl(Tag):
    """Defines a description list"""

    pass


class Dt(Tag):
    """Defines a term/name in a description list"""

    pass


class Em(Tag):
    """Defines emphasized text"""

    pass


class Embed(Tag):
    """Defines a container for an external application"""

    pass


class Fieldset(Tag):
    """Groups related elements in a form"""

    pass


class Figcaption(Tag):
    """Defines a caption for a <figure> element"""

    pass


class Figure(Tag):
    """Specifies self-contained content"""

    pass


class Footer(Tag):
    """Defines a footer for a document or section"""

    pass


class Form(Tag):
    """Defines an HTML form for user input"""

    pass


class H1(Tag):
    """H1 header"""

    pass


class H2(Tag):
    """H2 header"""

    pass


class H3(Tag):
    """H3 header"""

    pass


class H4(Tag):
    """H4 header"""

    pass


class H5(Tag):
    """H5 header"""

    pass


class H6(Tag):
    """H6 header"""

    pass


class Head(Tag):
    """Contains metadata/information for the document"""

    pass


class Header(Tag):
    """Defines a header for a document or section"""

    pass


class Hgroup(Tag):
    """Defines a header and related content"""

    pass


class Hr(Tag):
    """Defines a thematic change in the content"""

    pass


class I(Tag):  # noqa: E742
    """Defines a part of text in an alternate voice or mood"""

    pass


class Iframe(Tag):
    """Defines an inline frame"""

    pass


class Img(Tag):
    """Defines an image"""

    pass


class Input(Tag):
    """Defines an input control"""

    pass


class Ins(Tag):
    """Defines a text that has been inserted into a document"""

    pass


class Kbd(Tag):
    """Defines keyboard input"""

    pass


class Label(Tag):
    """Defines a label for an <input> element"""

    pass


class Legend(Tag):
    """Defines a caption for a <fieldset> element"""

    pass


class Li(Tag):
    """Defines a list item"""

    pass


class Link(Tag):
    """Defines the relationship between a document and an external resource (most used to link to style sheets)"""

    pass


class Main(Tag):
    """Specifies the main content of a document"""

    pass


class Map(Tag):
    """Defines an image map"""

    pass


class Mark(Tag):
    """Defines marked/highlighted text"""

    pass


class Menu(Tag):
    """Defines an unordered list"""

    pass


class Meta(Tag):
    """Defines metadata about an HTML document"""

    pass


class Meter(Tag):
    """Defines a scalar measurement within a known range (a gauge)"""

    pass


class Nav(Tag):
    """Defines navigation links"""

    pass


class Noscript(Tag):
    """Defines an alternate content for users that do not support client-side scripts"""

    pass


class Object(Tag):
    """Defines a container for an external application"""

    pass


class Ol(Tag):
    """Defines an ordered list"""

    pass


class Optgroup(Tag):
    """Defines a group of related options in a drop-down list"""

    pass


class Option(Tag):
    """Defines an option in a drop-down list"""

    pass


class Output(Tag):
    """Defines the result of a calculation"""

    pass


class P(Tag):
    """Defines a paragraph"""

    pass


class Param(Tag):
    """Defines a parameter for an object"""

    pass


class Picture(Tag):
    """Defines a container for multiple image resources"""

    pass


class Pre(Tag):
    """Defines preformatted text"""

    pass


class Progress(Tag):
    """Represents the progress of a task"""

    pass


class Q(Tag):
    """Defines a short quotation"""

    pass


class Rp(Tag):
    """Defines what to show in browsers that do not support ruby annotations"""

    pass


class Rt(Tag):
    """Defines an explanation/pronunciation of characters (for East Asian typography)"""

    pass


class Ruby(Tag):
    """Defines a ruby annotation (for East Asian typography)"""

    pass


class S(Tag):
    """Defines text that is no longer correct"""

    pass


class Samp(Tag):
    """Defines sample output from a computer program"""

    pass


class Script(Tag):
    """Defines a client-side script"""

    pass


class Search(Tag):
    """Defines a search section"""

    pass


class Section(Tag):
    """Defines a section in a document"""

    pass


class Select(Tag):
    """Defines a drop-down list"""

    pass


class Small(Tag):
    """Defines smaller text"""

    pass


class Source(Tag):
    """Defines multiple media resources for media elements (<video> and <audio>)"""

    pass


class Span(Tag):
    """Defines a section in a document"""

    pass


class Strong(Tag):
    """Defines important text"""

    pass


class Style(Tag):
    """Defines style information for a document"""

    pass


class Sub(Tag):
    """Defines subscripted text"""

    pass


class Summary(Tag):
    """Defines a visible heading for a <details> element"""

    pass


class Sup(Tag):
    """Defines superscripted text"""

    pass


class Table(Tag):
    """Defines a table"""

    pass


class Tbody(Tag):
    """Groups the body content in a table"""

    pass


class Td(Tag):
    """Defines a cell in a table"""

    pass


class Template(Tag):
    """Defines a container for content that should be hidden when the page loads"""

    pass


class Textarea(Tag):
    """Defines a multiline input control (text area)"""

    pass


class Tfoot(Tag):
    """Groups the footer content in a table"""

    pass


class Th(Tag):
    """Defines a header cell in a table"""

    pass


class Thead(Tag):
    """Groups the header content in a table"""

    pass


class Time(Tag):
    """Defines a specific time (or datetime)"""

    pass


class Title(Tag):
    """Defines a title for the document"""

    pass


class Tr(Tag):
    """Defines a row in a table"""

    pass


class Track(Tag):
    """Defines text tracks for media elements (<video> and <audio>)"""

    pass


class U(Tag):
    """Defines some text that is unarticulated and styled differently from normal text"""

    pass


class Ul(Tag):
    """Defines an unordered list"""

    pass


class Var(Tag):
    """Defines a variable"""

    pass


class Video(Tag):
    """Defines embedded video content"""

    pass


class Wbr(Tag):
    """Defines a possible line-break"""

    pass



from fastapi import FastAPI
from fastapi.testclient import TestClient
import air


def test_air_app_factory():
    app = air.Air()

    @app.get("/test")
    def test_endpoint():
        return air.H1("Hello, World!")

    client = TestClient(app)
    response = client.get("/test")

    assert response.status_code == 200
    assert response.headers["content-type"] == "text/html; charset=utf-8"
    assert response.text == "<h1>Hello, World!</h1>"


def test_air_plus_fastapi():
    app = FastAPI()
    html = air.Air()

    @app.get("/api")
    def test_api():
        return {"text": "hello, world"}

    @html.get("/page")
    def test_page():
        return air.H1("Hello, World!")

    @html.get("/page-html")
    def test_page_html():
        return "<h1>Hello, World!</h1>"

    # Combine into one app
    app.mount("/", html)

    client = TestClient(app)

    # Test the API
    response = client.get("/api")
    assert response.status_code == 200
    assert response.headers["content-type"] == "application/json"
    assert response.json() == {"text": "hello, world"}

    # Test the page
    response = client.get("/page")
    assert response.status_code == 200
    assert response.headers["content-type"] == "text/html; charset=utf-8"
    assert response.text == "<h1>Hello, World!</h1>"

    # Test the page with HTML
    response = client.get("/page-html")
    assert response.status_code == 200
    assert response.headers["content-type"] == "text/html; charset=utf-8"
    assert response.text == "<h1>Hello, World!</h1>"


def test_page_decorator():
    app = air.Air()
    page = app.page

    @page
    def index():
        return air.H1("Home page")

    @page
    def about():
        return "<h1>About page</h1>"

    client = TestClient(app)
    response = client.get("/")
    assert response.status_code == 200
    assert response.headers["content-type"] == "text/html; charset=utf-8"
    assert response.text == "<h1>Home page</h1>"

    response = client.get("/about")
    assert response.status_code == 200
    assert response.headers["content-type"] == "text/html; charset=utf-8"
    assert response.text == "<h1>About page</h1>"



"""A FastAPI-powered breath of fresh air in Python web development."""

__version__ = "0.9.0"

from .applications import Air as Air
from .requests import is_htmx_request as is_htmx_request
from .responses import AirResponse as AirResponse, TagResponse as TagResponse
from .templates import Jinja2Renderer as Jinja2Renderer
from .tags import Tag as Tag
from .tags import (
    A as A,
    Abbr as Abbr,
    Address as Address,
    Area as Area,
    Article as Article,
    Aside as Aside,
    Audio as Audio,
    B as B,
    Base as Base,
    Bdi as Bdi,
    Bdo as Bdo,
    Blockquote as Blockquote,
    Body as Body,
    Br as Br,
    Button as Button,
    Canvas as Canvas,
    Caption as Caption,
    Cite as Cite,
    Code as Code,
    Col as Col,
    Colgroup as Colgroup,
    Data as Data,
    Datalist as Datalist,
    Dd as Dd,
    Del as Del,
    Details as Details,
    Dfn as Dfn,
    Dialog as Dialog,
    Div as Div,
    Dl as Dl,
    Dt as Dt,
    Em as Em,
    Embed as Embed,
    Fieldset as Fieldset,
    Figcaption as Figcaption,
    Figure as Figure,
    Footer as Footer,
    Form as Form,
    H1 as H1,
    H2 as H2,
    H3 as H3,
    H4 as H4,
    H5 as H5,
    H6 as H6,
    Head as Head,
    Header as Header,
    Hgroup as Hgroup,
    Hr as Hr,
    Html as Html,
    I as I,
    Iframe as Iframe,
    Img as Img,
    Input as Input,
    Ins as Ins,
    Kbd as Kbd,
    Label as Label,
    Legend as Legend,
    Li as Li,
    Link as Link,
    Main as Main,
    Map as Map,
    Mark as Mark,
    Menu as Menu,
    Meta as Meta,
    Meter as Meter,
    Nav as Nav,
    Noscript as Noscript,
    Object as Object,
    Ol as Ol,
    Optgroup as Optgroup,
    Option as Option,
    Output as Output,
    P as P,
    Param as Param,
    Picture as Picture,
    Pre as Pre,
    Progress as Progress,
    Q as Q,
    RawHTML as RawHTML,
    Rp as Rp,
    Rt as Rt,
    Ruby as Ruby,
    S as S,
    Samp as Samp,
    Script as Script,
    Search as Search,
    Section as Section,
    Select as Select,
    Small as Small,
    Source as Source,
    Span as Span,
    Strong as Strong,
    Style as Style,
    Sub as Sub,
    Summary as Summary,
    Sup as Sup,
    Table as Table,
    Tbody as Tbody,
    Td as Td,
    Template as Template,
    Textarea as Textarea,
    Tfoot as Tfoot,
    Th as Th,
    Thead as Thead,
    Time as Time,
    Title as Title,
    Tr as Tr,
    Track as Track,
    U as U,
    Ul as Ul,
    Var as Var,
    Video as Video,
    Wbr as Wbr,
)
from . import svg as svg



from typing import (
    Any,
    Callable,
    Coroutine,
    Dict,
    List,
    Optional,
    Sequence,
    Type,
    TypeVar,
    Union,
)

from fastapi import routing
from fastapi.datastructures import Default
from fastapi.params import Depends
from starlette.middleware import Middleware
from starlette.requests import Request
from starlette.responses import Response
from starlette.routing import BaseRoute
from starlette.types import Lifespan
from typing_extensions import Annotated, Doc, deprecated
from fastapi import FastAPI
from .responses import AirResponse


AppType = TypeVar("AppType", bound="FastAPI")


class Air(FastAPI):
    """FastAPI wrapper class with AirResponse as the default response class."""

    def __init__(
        self: AppType,
        *,
        debug: Annotated[
            bool,
            Doc(
                """
                Boolean indicating if debug tracebacks should be returned on server
                errors.

                Read more in the
                [Starlette docs for Applications](https://www.starlette.io/applications/#instantiating-the-application).
                """
            ),
        ] = False,
        routes: Annotated[
            Optional[List[BaseRoute]],
            Doc(
                """
                **Note**: you probably shouldn't use this parameter, it is inherited
                from Starlette and supported for compatibility.

                ---

                A list of routes to serve incoming HTTP and WebSocket requests.
                """
            ),
            deprecated(
                """
                You normally wouldn't use this parameter with FastAPI, it is inherited
                from Starlette and supported for compatibility.

                In FastAPI, you normally would use the *path operation methods*,
                like `app.get()`, `app.post()`, etc.
                """
            ),
        ] = None,
        servers: Annotated[
            Optional[List[Dict[str, Union[str, Any]]]],
            Doc(
                """
                A `list` of `dict`s with connectivity information to a target server.

                You would use it, for example, if your application is served from
                different domains and you want to use the same Swagger UI in the
                browser to interact with each of them (instead of having multiple
                browser tabs open). Or if you want to leave fixed the possible URLs.

                If the servers `list` is not provided, or is an empty `list`, the
                default value would be a `dict` with a `url` value of `/`.

                Each item in the `list` is a `dict` containing:

                * `url`: A URL to the target host. This URL supports Server Variables
                and MAY be relative, to indicate that the host location is relative
                to the location where the OpenAPI document is being served. Variable
                substitutions will be made when a variable is named in `{`brackets`}`.
                * `description`: An optional string describing the host designated by
                the URL. [CommonMark syntax](https://commonmark.org/) MAY be used for
                rich text representation.
                * `variables`: A `dict` between a variable name and its value. The value
                    is used for substitution in the server's URL template.

                Read more in the
                [FastAPI docs for Behind a Proxy](https://fastapi.tiangolo.com/advanced/behind-a-proxy/#additional-servers).

                **Example**

                ```python
                from fastapi import FastAPI

                app = FastAPI(
                    servers=[
                        {"url": "https://stag.example.com", "description": "Staging environment"},
                        {"url": "https://prod.example.com", "description": "Production environment"},
                    ]
                )
                ```
                """
            ),
        ] = None,
        dependencies: Annotated[
            Optional[Sequence[Depends]],
            Doc(
                """
                A list of global dependencies, they will be applied to each
                *path operation*, including in sub-routers.

                Read more about it in the
                [FastAPI docs for Global Dependencies](https://fastapi.tiangolo.com/tutorial/dependencies/global-dependencies/).

                **Example**

                ```python
                from fastapi import Depends, FastAPI

                from .dependencies import func_dep_1, func_dep_2

                app = FastAPI(dependencies=[Depends(func_dep_1), Depends(func_dep_2)])
                ```
                """
            ),
        ] = None,
        default_response_class: Annotated[
            Type[Response],
            Doc(
                """
                The default response class to be used.

                Read more in the
                [FastAPI docs for Custom Response - HTML, Stream, File, others](https://fastapi.tiangolo.com/advanced/custom-response/#default-response-class).

                **Analogy**

                ```python
                from fastapi import FastAPI
                from air import AirResponse

                app = FastAPI(default_response_class=AirResponse)
                ```
                """
            ),
        ] = Default(AirResponse),
        redirect_slashes: Annotated[
            bool,
            Doc(
                """
                Whether to detect and redirect slashes in URLs when the client doesn't
                use the same format.

                **Example**

                ```python
                from fastapi import FastAPI

                app = FastAPI(redirect_slashes=True)  # the default

                @app.get("/items/")
                async def read_items():
                    return [{"item_id": "Foo"}]
                ```

                With this app, if a client goes to `/items` (without a trailing slash),
                they will be automatically redirected with an HTTP status code of 307
                to `/items/`.
                """
            ),
        ] = True,
        middleware: Annotated[
            Optional[Sequence[Middleware]],
            Doc(
                """
                List of middleware to be added when creating the application.

                In FastAPI you would normally do this with `app.add_middleware()`
                instead.

                Read more in the
                [FastAPI docs for Middleware](https://fastapi.tiangolo.com/tutorial/middleware/).
                """
            ),
        ] = None,
        exception_handlers: Annotated[
            Optional[
                Dict[
                    Union[int, Type[Exception]],
                    Callable[[Request, Any], Coroutine[Any, Any, Response]],
                ]
            ],
            Doc(
                """
                A dictionary with handlers for exceptions.

                In FastAPI, you would normally use the decorator
                `@app.exception_handler()`.

                Read more in the
                [FastAPI docs for Handling Errors](https://fastapi.tiangolo.com/tutorial/handling-errors/).
                """
            ),
        ] = None,
        on_startup: Annotated[
            Optional[Sequence[Callable[[], Any]]],
            Doc(
                """
                A list of startup event handler functions.

                You should instead use the `lifespan` handlers.

                Read more in the [FastAPI docs for `lifespan`](https://fastapi.tiangolo.com/advanced/events/).
                """
            ),
        ] = None,
        on_shutdown: Annotated[
            Optional[Sequence[Callable[[], Any]]],
            Doc(
                """
                A list of shutdown event handler functions.

                You should instead use the `lifespan` handlers.

                Read more in the
                [FastAPI docs for `lifespan`](https://fastapi.tiangolo.com/advanced/events/).
                """
            ),
        ] = None,
        lifespan: Annotated[
            Optional[Lifespan[AppType]],
            Doc(
                """
                A `Lifespan` context manager handler. This replaces `startup` and
                `shutdown` functions with a single context manager.

                Read more in the
                [FastAPI docs for `lifespan`](https://fastapi.tiangolo.com/advanced/events/).
                """
            ),
        ] = None,
        webhooks: Annotated[
            Optional[routing.APIRouter],
            Doc(
                """
                Add OpenAPI webhooks. This is similar to `callbacks` but it doesn't
                depend on specific *path operations*.

                It will be added to the generated OpenAPI (e.g. visible at `/docs`).

                **Note**: This is available since OpenAPI 3.1.0, FastAPI 0.99.0.

                Read more about it in the
                [FastAPI docs for OpenAPI Webhooks](https://fastapi.tiangolo.com/advanced/openapi-webhooks/).
                """
            ),
        ] = None,
        deprecated: Annotated[
            Optional[bool],
            Doc(
                """
                Mark all *path operations* as deprecated. You probably don't need it,
                but it's available.

                It will be added to the generated OpenAPI (e.g. visible at `/docs`).

                Read more about it in the
                [FastAPI docs for Path Operation Configuration](https://fastapi.tiangolo.com/tutorial/path-operation-configuration/).
                """
            ),
        ] = None,
        **extra: Annotated[
            Any,
            Doc(
                """
                Extra keyword arguments to be stored in the app, not used by FastAPI
                anywhere.
                """
            ),
        ],
    ) -> None:
        """Initialize Air app with AirResponse as default response class.

        This preserves all FastAPI initialization parameters while setting
        AirResponse as the default response class.
        """
        super().__init__(
            debug=debug,
            routes=routes,
            servers=servers,
            dependencies=dependencies,
            # TODO: Have this accept default_response_class=default_response_class
            default_response_class=AirResponse,
            middleware=middleware,
            exception_handlers=exception_handlers,
            on_startup=on_startup,
            on_shutdown=on_shutdown,
            lifespan=lifespan,
            webhooks=webhooks,
            deprecated=deprecated,
            **extra,
        )

    def page(self, func):
        """Decorator that creates a GET route using the function name as the path."""
        if func.__name__ == "index":
            route_name = "/"
        else:
            route_name = f"/{func.__name__}"
        return self.get(route_name)(func)



from .tags import CaseTag


class A(CaseTag):
    """Defines an SVG hyperlink"""

    pass


class Animate(CaseTag):
    """Defines animation on an SVG element"""

    pass


class AnimateMotion(CaseTag):
    """Defines how an element moves along a motion path"""

    pass


class AnimateTransform(CaseTag):
    """Animates transform attributes on an element"""

    pass


class Circle(CaseTag):
    """Defines a circle"""

    pass


class ClipPath(CaseTag):
    """Defines a clipping path"""

    pass


class Defs(CaseTag):
    """Defines reusable objects"""

    pass


class Desc(CaseTag):
    """Defines a description of an element"""

    pass


class Ellipse(CaseTag):
    """Defines an ellipse"""

    pass


class FeBlend(CaseTag):
    """Defines image blending"""

    pass


class FeColorMatrix(CaseTag):
    """Applies a matrix transformation on color values"""

    pass


class FeComponentTransfer(CaseTag):
    """Performs component-wise remapping of data"""

    pass


class FeComposite(CaseTag):
    """Performs image compositing"""

    pass


class FeConvolveMatrix(CaseTag):
    """Applies a matrix convolution filter"""

    pass


class FeDiffuseLighting(CaseTag):
    """Lights an image using diffuse lighting"""

    pass


class FeDisplacementMap(CaseTag):
    """Displaces an image using another image"""

    pass


class FeDistantLight(CaseTag):
    """Defines a distant light source"""

    pass


class FeDropShadow(CaseTag):
    """Creates a drop shadow effect"""

    pass


class FeFlood(CaseTag):
    """Fills the filter region with a color"""

    pass


class FeFuncA(CaseTag):
    """Defines the alpha transfer function"""

    pass


class FeFuncB(CaseTag):
    """Defines the blue transfer function"""

    pass


class FeFuncG(CaseTag):
    """Defines the green transfer function"""

    pass


class FeFuncR(CaseTag):
    """Defines the red transfer function"""

    pass


class FeGaussianBlur(CaseTag):
    """Applies Gaussian blur to an image"""

    pass


class FeImage(CaseTag):
    """Refers to an external image"""

    pass


class FeMerge(CaseTag):
    """Merges multiple filter nodes"""

    pass


class FeMergeNode(CaseTag):
    """Defines a node for feMerge"""

    pass


class FeMorphology(CaseTag):
    """Applies morphological operations"""

    pass


class FeOffset(CaseTag):
    """Offsets an image"""

    pass


class FePointLight(CaseTag):
    """Defines a point light source"""

    pass


class FeSpecularLighting(CaseTag):
    """Lights an image using specular lighting"""

    pass


class FeSpotLight(CaseTag):
    """Defines a spot light source"""

    pass


class FeTile(CaseTag):
    """Tiles an image to fill a rectangle"""

    pass


class FeTurbulence(CaseTag):
    """Creates turbulence noise"""

    pass


class Filter(CaseTag):
    """Defines a filter effect"""

    pass


class ForeignObject(CaseTag):
    """Allows inclusion of foreign XML"""

    pass


class G(CaseTag):
    """Groups SVG elements"""

    pass


class Image(CaseTag):
    """Embeds an image"""

    pass


class Line(CaseTag):
    """Defines a line"""

    pass


class LinearGradient(CaseTag):
    """Defines a linear gradient"""

    pass


class Marker(CaseTag):
    """Defines a graphic for drawing on lines"""

    pass


class Mask(CaseTag):
    """Defines a mask"""

    pass


class Metadata(CaseTag):
    """Defines metadata"""

    pass


class Mpath(CaseTag):
    """Defines a motion path"""

    pass


class Path(CaseTag):
    """Defines a path"""

    pass


class Pattern(CaseTag):
    """Defines a pattern"""

    pass


class Polygon(CaseTag):
    """Defines a polygon"""

    pass


class Polyline(CaseTag):
    """Defines a polyline"""

    pass


class RadialGradient(CaseTag):
    """Defines a radial gradient"""

    pass


class Rect(CaseTag):
    """Defines a rectangle"""

    pass


class Script(CaseTag):
    """Defines a script"""

    pass


class Set(CaseTag):
    """Sets an attribute value"""

    pass


class Stop(CaseTag):
    """Defines a gradient stop"""

    pass


class Style(CaseTag):
    """Defines style information"""

    pass


class Svg(CaseTag):
    """Defines an SVG document fragment"""

    pass


class Switch(CaseTag):
    """Defines conditional processing"""

    pass


class Symbol(CaseTag):
    """Defines a reusable symbol"""

    pass


class Text(CaseTag):
    """Defines text content"""

    pass


class TextPath(CaseTag):
    """Defines text along a path"""

    pass


class Title(CaseTag):
    """Defines a title for the SVG document"""

    pass


class Tspan(CaseTag):
    """Defines a text span"""

    pass


class Use(CaseTag):
    """References another element"""

    pass


class View(CaseTag):
    """Defines a view"""

    pass
